@@ -1,4 +1,23 @@
-/* =======================================================================
+/*
+ * Copyright (C) 1997-2001 Id Software, Inc.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or (at
+ * your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *
+ * See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ *
+ * =======================================================================
  *
  * The insane earth soldiers.
  *
@@ -15,6 +34,11 @@ static int sound_shake;
 static int sound_moan;
 static int sound_scream[8];
 
+static int  sound_step;
+static int  sound_step2;
+static int  sound_step3;
+static int  sound_step4;
+
 void insane_stand(edict_t *self);
 void insane_dead(edict_t *self);
 void insane_cross(edict_t *self);
@@ -24,10 +48,47 @@ void insane_checkdown(edict_t *self);
 void insane_checkup(edict_t *self);
 void insane_onground(edict_t *self);
 
+void
+insane_footstep(edict_t *self)
+{
+	if (!g_monsterfootsteps->value)
+		return;
+
+	// Lazy loading for savegame compatibility.
+	if (sound_step == 0 || sound_step2 == 0 || sound_step3 == 0 || sound_step4 == 0)
+	{
+		sound_step = gi.soundindex("player/step1.wav");
+		sound_step2 = gi.soundindex("player/step2.wav");
+		sound_step3 = gi.soundindex("player/step3.wav");
+		sound_step4 = gi.soundindex("player/step4.wav");
+	}
+
+	int     i;
+	i = randk() % 4;
+
+	if (i == 0)
+	{
+		gi.sound(self, CHAN_BODY, sound_step, 0.7, ATTN_NORM, 0);
+	}
+	else if (i == 1)
+	{
+		gi.sound(self, CHAN_BODY, sound_step2, 0.7, ATTN_NORM, 0);
+	}
+	else if (i == 2)
+	{
+		gi.sound(self, CHAN_BODY, sound_step3, 0.7, ATTN_NORM, 0);
+	}
+	else if (i == 3)
+	{
+		gi.sound(self, CHAN_BODY, sound_step4, 0.7, ATTN_NORM, 0);
+	}
+}
+
+
 void
 insane_fist(edict_t *self)
 {
-  	if (!self)
+	if (!self)
 	{
 		return;
 	}
@@ -38,7 +99,7 @@ insane_fist(edict_t *self)
 void
 insane_shake(edict_t *self)
 {
-  	if (!self)
+	if (!self)
 	{
 		return;
 	}
@@ -49,7 +110,7 @@ insane_shake(edict_t *self)
 void
 insane_moan(edict_t *self)
 {
-  	if (!self)
+	if (!self)
 	{
 		return;
 	}
@@ -80,7 +141,7 @@ insane_scream(edict_t *self)
 	gi.sound(self, CHAN_VOICE, sound_scream[rand() % 8], 1, ATTN_IDLE, 0);
 }
 
-mframe_t insane_frames_stand_normal[] = {
+static mframe_t insane_frames_stand_normal[] = {
 	{ai_stand, 0, NULL},
 	{ai_stand, 0, NULL},
 	{ai_stand, 0, NULL},
@@ -89,14 +150,15 @@ mframe_t insane_frames_stand_normal[] = {
 	{ai_stand, 0, insane_checkdown}
 };
 
-mmove_t insane_move_stand_normal = {
+mmove_t insane_move_stand_normal =
+{
 	FRAME_stand60,
-   	FRAME_stand65,
+	FRAME_stand65,
    	insane_frames_stand_normal,
    	insane_stand
 };
 
-mframe_t insane_frames_stand_insane[] = {
+static mframe_t insane_frames_stand_insane[] = {
 	{ai_stand, 0, insane_shake},
 	{ai_stand, 0, NULL},
 	{ai_stand, 0, NULL},
@@ -129,14 +191,15 @@ mframe_t insane_frames_stand_insane[] = {
 	{ai_stand, 0, insane_checkdown}
 };
 
-mmove_t insane_move_stand_insane = {
+mmove_t insane_move_stand_insane =
+{
 	FRAME_stand65,
    	FRAME_stand94,
-   	insane_frames_stand_insane,
+	insane_frames_stand_insane,
    	insane_stand
 };
 
-mframe_t insane_frames_uptodown[] = {
+static mframe_t insane_frames_uptodown[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -182,14 +245,15 @@ mframe_t insane_frames_uptodown[] = {
 	{ai_move, 0, NULL}
 };
 
-mmove_t insane_move_uptodown = {
+mmove_t insane_move_uptodown =
+{
 	FRAME_stand1,
    	FRAME_stand40,
    	insane_frames_uptodown,
    	insane_onground
 };
 
-mframe_t insane_frames_downtoup[] = {
+static mframe_t insane_frames_downtoup[] = {
 	{ai_move, -0.7, NULL}, /* 41 */
 	{ai_move, -1.2, NULL}, /* 42 */
 	{ai_move, -1.5, NULL}, /* 43 */
@@ -208,17 +272,18 @@ mframe_t insane_frames_downtoup[] = {
 	{ai_move, -0.3, NULL}, /* 56 */
 	{ai_move, 0, NULL}, /* 57 */
 	{ai_move, 0, NULL}, /* 58 */
-	{ai_move, 0, NULL} /* 59 */
+	{ai_move, 0, NULL}  /* 59 */
 };
 
-mmove_t insane_move_downtoup = {
+mmove_t insane_move_downtoup =
+{
 	FRAME_stand41,
-   	FRAME_stand59,
-   	insane_frames_downtoup,
-   	insane_stand
+	FRAME_stand59,
+	insane_frames_downtoup,
+	insane_stand
 };
 
-mframe_t insane_frames_jumpdown[] = {
+static mframe_t insane_frames_jumpdown[] = {
 	{ai_move, 0.2, NULL},
 	{ai_move, 11.5, NULL},
 	{ai_move, 5.1, NULL},
@@ -226,14 +291,15 @@ mframe_t insane_frames_jumpdown[] = {
 	{ai_move, 0, NULL}
 };
 
-mmove_t insane_move_jumpdown = {
+mmove_t insane_move_jumpdown =
+{
 	FRAME_stand96,
-   	FRAME_stand100,
+	FRAME_stand100,
    	insane_frames_jumpdown,
    	insane_onground
 };
 
-mframe_t insane_frames_down[] = {
+static mframe_t insane_frames_down[] = {
 	{ai_move, 0, NULL}, /* 100 */
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -297,92 +363,97 @@ mframe_t insane_frames_down[] = {
 	{ai_move, 0, insane_checkup} /* 160 */
 };
 
-mmove_t insane_move_down = {
+mmove_t insane_move_down =
+{
 	FRAME_stand100,
-   	FRAME_stand160,
+	FRAME_stand160,
    	insane_frames_down,
    	insane_onground
 };
 
-mframe_t insane_frames_walk_normal[] = {
+static mframe_t insane_frames_walk_normal[] = {
 	{ai_walk, 0, insane_scream},
 	{ai_walk, 2.5, NULL},
 	{ai_walk, 3.5, NULL},
 	{ai_walk, 1.7, NULL},
 	{ai_walk, 2.3, NULL},
 	{ai_walk, 2.4, NULL},
-	{ai_walk, 2.2, NULL},
+	{ai_walk, 2.2, insane_footstep},
 	{ai_walk, 4.2, NULL},
 	{ai_walk, 5.6, NULL},
 	{ai_walk, 3.3, NULL},
 	{ai_walk, 2.4, NULL},
 	{ai_walk, 0.9, NULL},
-	{ai_walk, 0, NULL}
+	{ai_walk, 0, insane_footstep}
 };
 
-mmove_t insane_move_walk_normal = {
+mmove_t insane_move_walk_normal =
+{
+	FRAME_walk27,
+	FRAME_walk39,
+	insane_frames_walk_normal,
+	insane_walk
+};
+
+mmove_t insane_move_run_normal =
+{
 	FRAME_walk27,
    	FRAME_walk39,
    	insane_frames_walk_normal,
-   	insane_walk
+	insane_run
 };
 
-mmove_t insane_move_run_normal = {
-	FRAME_walk27,
-   	FRAME_walk39,
-   	insane_frames_walk_normal,
-   	insane_run
-};
-
-mframe_t insane_frames_walk_insane[] = {
+static mframe_t insane_frames_walk_insane[] = {
 	{ai_walk, 0, insane_scream}, /* walk 1 */
 	{ai_walk, 3.4, NULL}, /* walk 2 */
 	{ai_walk, 3.6, NULL}, /* 3 */
 	{ai_walk, 2.9, NULL}, /* 4 */
 	{ai_walk, 2.2, NULL}, /* 5 */
 	{ai_walk, 2.6, NULL}, /* 6 */
-	{ai_walk, 0, NULL}, /* 7 */
+	{ai_walk, 0, insane_footstep}, /* 7 */
 	{ai_walk, 0.7, NULL}, /* 8 */
 	{ai_walk, 4.8, NULL}, /* 9 */
 	{ai_walk, 5.3, NULL}, /* 10 */
 	{ai_walk, 1.1, NULL}, /* 11 */
-	{ai_walk, 2, NULL}, /* 12 */
+	{ai_walk, 2, insane_footstep}, /* 12 */
 	{ai_walk, 0.5, NULL}, /* 13 */
 	{ai_walk, 0, NULL}, /* 14 */
 	{ai_walk, 0, NULL}, /* 15 */
 	{ai_walk, 4.9, NULL}, /* 16 */
 	{ai_walk, 6.7, NULL}, /* 17 */
 	{ai_walk, 3.8, NULL}, /* 18 */
-	{ai_walk, 2, NULL}, /* 19 */
+	{ai_walk, 2, insane_footstep}, /* 19 */
 	{ai_walk, 0.2, NULL}, /* 20 */
 	{ai_walk, 0, NULL}, /* 21 */
 	{ai_walk, 3.4, NULL}, /* 22 */
 	{ai_walk, 6.4, NULL}, /* 23 */
 	{ai_walk, 5, NULL}, /* 24 */
-	{ai_walk, 1.8, NULL}, /* 25 */
+	{ai_walk, 1.8, insane_footstep}, /* 25 */
 	{ai_walk, 0, NULL} /* 26 */
 };
 
-mmove_t insane_move_walk_insane = {
+mmove_t insane_move_walk_insane =
+{
 	FRAME_walk1,
-	FRAME_walk26,
+   	FRAME_walk26,
    	insane_frames_walk_insane,
    	insane_walk
 };
 
-mmove_t insane_move_run_insane = {
+mmove_t insane_move_run_insane =
+{
 	FRAME_walk1,
-   	FRAME_walk26,
+	FRAME_walk26,
    	insane_frames_walk_insane,
    	insane_run
 };
 
-mframe_t insane_frames_stand_pain[] = {
-	{ai_move, 0, NULL},
+static mframe_t insane_frames_stand_pain[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
+	{ai_move, 0, insane_footstep},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -391,14 +462,15 @@ mframe_t insane_frames_stand_pain[] = {
 	{ai_move, 0, NULL}
 };
 
-mmove_t insane_move_stand_pain = {
+mmove_t insane_move_stand_pain =
+{
 	FRAME_st_pain2,
    	FRAME_st_pain12,
    	insane_frames_stand_pain,
    	insane_run
 };
 
-mframe_t insane_frames_stand_death[] = {
+static mframe_t insane_frames_stand_death[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -418,14 +490,15 @@ mframe_t insane_frames_stand_death[] = {
 	{ai_move, 0, NULL}
 };
 
-mmove_t insane_move_stand_death = {
+mmove_t insane_move_stand_death =
+{
 	FRAME_st_death2,
    	FRAME_st_death18,
    	insane_frames_stand_death,
-   	insane_dead
+	insane_dead
 };
 
-mframe_t insane_frames_crawl[] = {
+static mframe_t insane_frames_crawl[] = {
 	{ai_walk, 0, insane_scream},
 	{ai_walk, 1.5, NULL},
 	{ai_walk, 2.1, NULL},
@@ -437,21 +510,23 @@ mframe_t insane_frames_crawl[] = {
 	{ai_walk, 2.4, NULL}
 };
 
-mmove_t insane_move_crawl = {
+mmove_t insane_move_crawl =
+{
 	FRAME_crawl1,
    	FRAME_crawl9,
    	insane_frames_crawl,
    	NULL
 };
 
-mmove_t insane_move_runcrawl = {
+mmove_t insane_move_runcrawl =
+{
 	FRAME_crawl1,
    	FRAME_crawl9,
-   	insane_frames_crawl,
+	insane_frames_crawl,
    	NULL
 };
 
-mframe_t insane_frames_crawl_pain[] = {
+static mframe_t insane_frames_crawl_pain[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -463,14 +538,15 @@ mframe_t insane_frames_crawl_pain[] = {
 	{ai_move, 0, NULL}
 };
 
-mmove_t insane_move_crawl_pain = {
+mmove_t insane_move_crawl_pain =
+{
 	FRAME_cr_pain2,
    	FRAME_cr_pain10,
    	insane_frames_crawl_pain,
    	insane_run
 };
 
-mframe_t insane_frames_crawl_death[] = {
+static mframe_t insane_frames_crawl_death[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -480,14 +556,15 @@ mframe_t insane_frames_crawl_death[] = {
 	{ai_move, 0, NULL}
 };
 
-mmove_t insane_move_crawl_death = {
+mmove_t insane_move_crawl_death =
+{
 	FRAME_cr_death10,
-   	FRAME_cr_death16,
+	FRAME_cr_death16,
    	insane_frames_crawl_death,
    	insane_dead
 };
 
-mframe_t insane_frames_cross[] = {
+static mframe_t insane_frames_cross[] = {
 	{ai_move, 0, insane_moan},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -505,14 +582,15 @@ mframe_t insane_frames_cross[] = {
 	{ai_move, 0, NULL}
 };
 
-mmove_t insane_move_cross = {
+mmove_t insane_move_cross =
+{
 	FRAME_cross1,
-   	FRAME_cross15,
+	FRAME_cross15,
    	insane_frames_cross,
    	insane_cross
 };
 
-mframe_t insane_frames_struggle_cross[] = {
+static mframe_t insane_frames_struggle_cross[] = {
 	{ai_move, 0, insane_scream},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -530,17 +608,18 @@ mframe_t insane_frames_struggle_cross[] = {
 	{ai_move, 0, NULL}
 };
 
-mmove_t insane_move_struggle_cross = {
+mmove_t insane_move_struggle_cross =
+{
 	FRAME_cross16,
    	FRAME_cross30,
-   	insane_frames_struggle_cross,
-   	insane_cross
+	insane_frames_struggle_cross,
+	insane_cross
 };
 
 void
 insane_cross(edict_t *self)
 {
-  	if (!self)
+	if (!self)
 	{
 		return;
 	}
@@ -558,7 +637,7 @@ insane_cross(edict_t *self)
 void
 insane_walk(edict_t *self)
 {
-  	if (!self)
+	if (!self)
 	{
 		return;
 	}
@@ -590,7 +669,7 @@ insane_walk(edict_t *self)
 void
 insane_run(edict_t *self)
 {
-  	if (!self)
+	if (!self)
 	{
 		return;
 	}
@@ -625,7 +704,7 @@ insane_pain(edict_t *self, edict_t *other /* unused */,
 {
 	int l, r;
 
-  	if (!self)
+	if (!self)
 	{
 		return;
 	}
@@ -676,7 +755,8 @@ insane_pain(edict_t *self, edict_t *other /* unused */,
 
 	if (((self->s.frame >= FRAME_crawl1) &&
 		 (self->s.frame <= FRAME_crawl9)) ||
-		((self->s.frame >= FRAME_stand99) && (self->s.frame <= FRAME_stand160)))
+		((self->s.frame >= FRAME_stand99) &&
+		 (self->s.frame <= FRAME_stand160)))
 	{
 		self->monsterinfo.currentmove = &insane_move_crawl_pain;
 	}
@@ -689,7 +769,7 @@ insane_pain(edict_t *self, edict_t *other /* unused */,
 void
 insane_onground(edict_t *self)
 {
-  	if (!self)
+	if (!self)
 	{
 		return;
 	}
@@ -700,7 +780,7 @@ insane_onground(edict_t *self)
 void
 insane_checkdown(edict_t *self)
 {
-  	if (!self)
+	if (!self)
 	{
 		return;
 	}
@@ -726,11 +806,12 @@ insane_checkdown(edict_t *self)
 void
 insane_checkup(edict_t *self)
 {
-  	if (!self)
+	if (!self)
 	{
 		return;
 	}
 
+	/* If Hold_Ground and Crawl are set */
 	if ((self->spawnflags & 4) && (self->spawnflags & 16))
 	{
 		return;
@@ -745,7 +826,7 @@ insane_checkup(edict_t *self)
 void
 insane_stand(edict_t *self)
 {
-  	if (!self)
+	if (!self)
 	{
 		return;
 	}
@@ -755,6 +836,7 @@ insane_stand(edict_t *self)
 		self->monsterinfo.currentmove = &insane_move_cross;
 		self->monsterinfo.aiflags |= AI_STAND_GROUND;
 	}
+	/* If Hold_Ground and Crawl are set */
 	else if ((self->spawnflags & 4) && (self->spawnflags & 16))
 	{
 		self->monsterinfo.currentmove = &insane_move_down;
@@ -773,7 +855,7 @@ insane_stand(edict_t *self)
 void
 insane_dead(edict_t *self)
 {
-  	if (!self)
+	if (!self)
 	{
 		return;
 	}
@@ -801,7 +883,7 @@ insane_die(edict_t *self, edict_t *inflictor /* unused */,
 {
 	int n;
 
-  	if (!self)
+	if (!self)
 	{
 		return;
 	}
@@ -812,15 +894,18 @@ insane_die(edict_t *self, edict_t *inflictor /* unused */,
 
 		for (n = 0; n < 2; n++)
 		{
-			ThrowGib(self, "models/objects/gibs/bone/tris.md2", damage, GIB_ORGANIC);
+			ThrowGib(self, "models/objects/gibs/bone/tris.md2",
+					damage, GIB_ORGANIC);
 		}
 
 		for (n = 0; n < 4; n++)
 		{
-			ThrowGib(self, "models/objects/gibs/sm_meat/tris.md2", damage, GIB_ORGANIC);
+			ThrowGib(self, "models/objects/gibs/sm_meat/tris.md2",
+					damage, GIB_ORGANIC);
 		}
 
-		ThrowHead(self, "models/objects/gibs/head2/tris.md2", damage, GIB_ORGANIC);
+		ThrowHead(self, "models/objects/gibs/head2/tris.md2",
+				damage, GIB_ORGANIC);
 		self->deadflag = DEAD_DEAD;
 		return;
 	}
@@ -862,7 +947,7 @@ insane_die(edict_t *self, edict_t *inflictor /* unused */,
 void
 SP_misc_insane(edict_t *self)
 {
-  	if (!self)
+	if (!self)
 	{
 		return;
 	}
@@ -873,6 +958,13 @@ SP_misc_insane(edict_t *self)
 		return;
 	}
 
+	// Force recaching at next footstep to ensure
+	// that the sound indices are correct.
+	sound_step = 0;
+	sound_step2 = 0;
+	sound_step3 = 0;
+	sound_step4 = 0;
+
 	sound_fist = gi.soundindex("insane/insane11.wav");
 	sound_shake = gi.soundindex("insane/insane5.wav");
 	sound_moan = gi.soundindex("insane/insane7.wav");
