@@ -256,7 +256,7 @@ InitGame(void)
 	aimfix = gi.cvar("aimfix", "0", CVAR_ARCHIVE);
 	g_machinegun_norecoil = gi.cvar("g_machinegun_norecoil", "0", CVAR_ARCHIVE);
 	g_quick_weap = gi.cvar("g_quick_weap", "1", CVAR_ARCHIVE);
-	g_swap_speed = gi.cvar("g_swap_speed", "1", 0);
+	g_swap_speed = gi.cvar("g_swap_speed", "1", CVAR_ARCHIVE);
 
 	/* items */
 	InitItems ();
